@@ -105,7 +105,10 @@ validConfigParams = collections.OrderedDict({
     'numJobs'                     : {'mandatory': False, 'type': int,                        'default': None},
     'inputParamsNames'            : {'mandatory': False, 'type': list, 'element_type': str,  'default': []},
     'inputParams'                 : {'mandatory': False, 'type': list, 'element_type': list, 'default': []},
-    'payload'                     : {'mandatory': True,  'type': str,                        'default': None},
+    'doCheckpointing'             : {'mandatory': False, 'type': bool,                       'default': False},
+    'checkpointSignal'            : {'mandatory': False, 'type': str,                        'default': ""},
+    'checkpointSecsBeforeTimeout' : {'mandatory': False, 'type': int,                        'default': 0},
+    'payload'                     : {'mandatory': True,  'type': str,                        'default': ""},
 })
 
 renamedParams = [
@@ -478,6 +481,24 @@ def validateConfigAfterDefaults(config):
         msg = invalidConfigMsg
         msg += "\nParameter 'stageoutLogs' is True, but parameter 'stageoutLogsDir' was not specified."
         raise ConfigurationException(msg)
+    # For checkpointing, accept only SIGUSR1/2 signals.
+    if config.doCheckpointing:
+        if config.checkpointSignal not in ['SIGUSR1', 'SIGUSR2', '']:
+            msg = "ERROR: Invalid Configuration:"
+            msg += "\nParameter 'checkpointSignal' has an invalid value: %s" % (config.checkpointSignal)
+            msg += "\nAllowed values are: SIGUSR1, SIGUSR2."
+            msg += "\nIt can also be set to an empty string to disable checkpoint-and-exit under a system signal."
+            raise ConfigurationException(msg)
+    # For checkpointing, set negative intervals in seconds to 0.
+    if config.doCheckpointing:
+        setattr(config, 'checkpointSecsBeforeTimeout', max(getattr(config, 'checkpointSecsBeforeTimeout'), 0))
+    # If 'checkpointSecsBeforeTimeout' is given, then 'checkpointSignal' should be specified.
+    if config.doCheckpointing:
+        if config.checkpointSecsBeforeTimeout > 0:
+            if not config.checkpointSignal:
+                msg = "ERROR: Invalid Configuration:"
+                msg += "\nParameter 'checkpointSecsBeforeTimeout' is set (greater than 0), but parameter 'checkpointSignal' was not specified."
+                raise ConfigurationException(msg)
 
 
 def fixConfigDirs(config):
