@@ -199,6 +199,9 @@ class SubmitWorker:
         numJobs                     = self.config.numJobs
         inputParamsNames            = self.config.inputParamsNames
         inputParams                 = self.config.inputParams
+        doCheckpointing             = self.config.doCheckpointing
+        checkpointSignal            = self.config.checkpointSignal
+        checkpointSecsBeforeTimeout = self.config.checkpointSecsBeforeTimeout
         payload                     = self.config.payload
 
         if writeLogsOnWN:
@@ -407,6 +410,11 @@ class SubmitWorker:
             batchScriptTemplate += \
 """\
 #SBATCH --error=__SBATCH_ERROR__
+"""
+        if doCheckpointing and checkpointSecsBeforeTimeout > 0:
+            batchScriptTemplate += \
+"""\
+#SBATCH --signal=B:__CHECKPOINT_SIGNAL_SHORT__@__CHECKPOINT_SECS_BEFORE_TIMEOUT__
 """
         if sbatch_additionalOptions:
             batchScriptTemplate += \
@@ -692,6 +700,27 @@ fi
 #==================================================================================
 # Read the job input arguments (if any) and run the payload
 #==================================================================================
+"""
+        if doCheckpointing and checkpointSignal:
+            batchScriptTemplate += \
+"""\
+
+function checkpoint() {
+    echo "Checkpointing is left up to the user ..."
+}
+
+function kill_payload() {
+    echo "Killing payload with signal __CHECKPOINT_SIGNAL__."
+    kill -s __CHECKPOINT_SIGNAL__ ${payloadpid}
+    wait ${payloadpid}
+    pec=$?
+}
+
+function signalhandler() {
+    echo "Received signal __CHECKPOINT_SIGNAL__."
+    checkpoint
+    kill_payload
+}
 """
         if setInputParamsCode:
             batchScriptTemplate += \
@@ -703,6 +732,11 @@ then
     __SET_INPUT_PARAMS_CODE__
     echo "==== Finished read of input parameters for job's payload (`date`) ===="
 fi
+"""
+        if doCheckpointing and checkpointSignal:
+            batchScriptTemplate += \
+"""\
+trap "signalhandler" __CHECKPOINT_SIGNAL__
 """
         batchScriptTemplate += \
 """\
@@ -713,8 +747,10 @@ then
     echo "------------------------- Begin payload output ------------------------"
     (
 __PAYLOAD__
-    )
-    pec=$?
+    ) &
+    payloadpid=$!
+    wait ${payloadpid}
+    pec=${pec:-$?}
     echo "--------------------------- End payload output ------------------------"
     echo "Payload exit code: ${pec}"
     if [[ ${pec} -ne 0 ]]
@@ -1010,6 +1046,10 @@ exit ${exitCode}
             '__STAGEOUT_FILES__': '" "'.join(stageoutFiles),
             '__STAGEOUT_LOGS__': str(stageoutLogs).lower(),
             '__STAGEOUT_LOGS_DIR__': stageoutLogsDir,
+            '__CHECKPOINT_SECS_BEFORE_TIMEOUT__': str(checkpointSecsBeforeTimeout),
+            '__CHECKPOINT_SIGNAL__': checkpointSignal,
+            '__CHECKPOINT_SIGNAL_SHORT__': checkpointSignal.replace('SIG', ''),
+
         }
 
         for j in range(numJobs):
